#include "mbed.h"
#include "SDFileSystem.h"
#include "wave_player.h"
#include "uLCD_4DGL.h"

//setup some color objects in flash using const's

#include "rtos.h"
#include "Small_6.h"
#include "Small_7.h"
#include "Arial_9.h"
#include "stdio.h"
#include "C12832_lcd.h"

#include "bubbles.h"
#include "songs.h"

//class for 3 PWM color values for RGBLED
class LEDColor
{
public:
    LEDColor(float r, float g, float b);
    float red;
    float green;
    float blue;
};
LEDColor:: LEDColor(float r, float g, float b)
    : red(r), green(g), blue(b)
{
}
//Operator overload to adjust brightness with no color change
LEDColor operator * (const LEDColor& x, const float& b)
{
    return LEDColor(x.red*b,x.green*b,x.blue*b);
}
//Operator overload to add colors
LEDColor operator + (const LEDColor& x, const LEDColor& y)
{
    return LEDColor(x.red+y.red,x.green+y.green,x.blue+y.blue);
}
 
//Class to control an RGB LED using three PWM pins
class RGBLed
{
public:
    RGBLed(PinName redpin, PinName greenpin, PinName bluepin);
    void write(float red,float green, float blue);
    void write(LEDColor c);
    RGBLed operator = (LEDColor c) {
        write(c);
        return *this;
    };
private:
    PwmOut _redpin;
    PwmOut _greenpin;
    PwmOut _bluepin;
};
 
RGBLed::RGBLed (PinName redpin, PinName greenpin, PinName bluepin)
    : _redpin(redpin), _greenpin(greenpin), _bluepin(bluepin)
{
    //50Hz PWM clock default a bit too low, go to 2000Hz (less flicker)
    _redpin.period(0.0005);
}
 
void RGBLed::write(float red,float green, float blue)
{
    _redpin = red;
    _greenpin = green;
    _bluepin = blue;
}
void RGBLed::write(LEDColor c)
{
    _redpin = c.red;
    _greenpin = c.green;
    _bluepin = c.blue;
}
 
//classes could be moved to include file
 
//Setup RGB led using PWM pins and class
RGBLed myRGBled(p24,p23,p22); //RGB PWM pins
const LEDColor red(1.0,0.0,0.0);
const LEDColor green(0.0,0.2,0.0);
//brighter green LED is scaled down to same as red and
//blue LED outputs on Sparkfun RGBLED
const LEDColor blue(0.0,0.0,1.0);
const LEDColor yellow(1.0,0.2,0.0);
const LEDColor white(1.0,0.2,1.0);
const LEDColor black(0.0,0.0,0.0);
char bred=0;
char bgreen=0;
char bblue=0; 

volatile bool songselect = false;
volatile bool homescreen = true;
volatile bool songchange = false;
uLCD_4DGL uLCD(p28,p27,p30); 
SDFileSystem sd(p5, p6, p7, p8, "sd"); //SD card
DigitalOut myled(LED1);
DigitalIn pb1(p20);
DigitalIn pb2(p19);
AnalogOut DACout(p18);
wave_player waver(&DACout);
Thread thread1, thread2, thread3;

// mutex to make the lcd lib thread safe
Mutex lcd_mutex;
int songnum = 1;
AnalogIn joy_pot(p16);

// Thread 1
// print homescreen to LCD
//new homescreen that doesn't have the flashing 
void homescreen_thread()
{
    while(true) {       // thread loop
        lcd_mutex.lock();
        if (homescreen){
            uLCD.cls();
            uLCD.text_height(1.3);
            uLCD.text_width(1.9);
            uLCD.color(WHITE);
            uLCD.locate(5,0);
            uLCD.printf("Tap Tap");
            uLCD.locate(4,1);
            uLCD.printf("Revolution");
            uLCD.locate(0,3);
            uLCD.printf("Pick a song");
            uLCD.text_height(1.3);
            uLCD.text_width(1.9);
            uLCD.locate(3,5);
            uLCD.printf("Fireflies");
            uLCD.locate(3,7);
            uLCD.printf("The Middle");
            uLCD.locate(3,9);
            uLCD.printf("Stacy's Mom");
            uLCD.locate(3,11);
            uLCD.printf("I Write Sins \n \t Not Tragedies");
            uLCD.filled_circle(6, 43, 4, GREEN); //new selection with circle instead of rectangle
            homescreen = false;}
        if (songchange){
            for (int i = 0; i < 4; i++){
            uLCD.filled_circle(6, i*15+43, 4,BLACK);
            }
            uLCD.filled_circle(6, (songnum-1)*15+43, 4,GREEN);
            songchange = false;
            }
            lcd_mutex.unlock();
            Thread::wait(200);
    }
}

// Thread 2
//for song selection during homescreen

void joystick_thread()
{
   while(1){
            if ((joy_pot <= (1.4/3.3)) && songnum>1) {
                songnum--;
                songchange = true;
                }
            else if ((joy_pot >= (1.8/3.3)) && songnum<4){
                songnum++;
                songchange = true;
                }
        Thread::wait(250);
    }
}

// Thread 3
//pb1 is to select song
//pb2 is to return to homescreen
void pbcontrol_thread()
{
    pb1.mode(PullUp);
    pb2.mode(PullUp);
    while(true) {       // thread loop
        if (!pb2)
            {
            homescreen = true;
            songselect = false;
            }
        if (!pb1)
            {
            songselect = true;
            }
        Thread::wait(100);   // value of pot1 / 100
    }
}

int main()
{
        //timestamp in terms of seconds
    
    //doesnt work yet, nee,d to ask hamblen
    uLCD.media_init();
    uLCD.set_sector_address(0x001D, 0x4C01);
    uLCD.display_image(0,0);
    thread1.start(homescreen_thread);
   thread2.start(joystick_thread);
    thread3.start(pbcontrol_thread);
    //startup sound
    myRGBled = blue; //tested to make sure led works, we can use whatever color(s) here
    FILE *wave_file;
    wave_file=fopen("/sd/cheer.wav","r");
    waver.play(wave_file);
    fclose(wave_file);
    while(1) 
    {
        
        if (songselect){
                myled = 0;
                homescreen = false;
                lcd_mutex.lock();
                uLCD.cls();
                uLCD.printf("You selected song %2d",songnum);
                lcd_mutex.unlock();
                //add case statement based on songnum or something
                //code for playing song from sd
                switch (songnum)
                {
                case 1:
                {
                FILE *wave_file;
                wave_file =fopen("/sd/4180 final project/Fireflies.wav", "r");
                waver.play(wave_file);
                fclose(wave_file);
                break;
                }
                case 2:
                {
                FILE *wave_file;
                wave_file =fopen("/sd/4180 final project/The_Middle.wav", "r");
                waver.play(wave_file);
                fclose(wave_file);
                break;
                }
                case 3:
                {
                FILE *wave_file;
                wave_file =fopen("/sd/4180 final project/Stacy's_Mom.wav", "r");
                waver.play(wave_file);
                fclose(wave_file);
                break;
                }
                case 4:
                {
                FILE *wave_file;
                wave_file =fopen("/sd/4180 final project/Sins_!_Tragedies.wav", "r");
                waver.play(wave_file);
                fclose(wave_file);
                break;
                }
                default:
                break;
            }
                
                }
        Thread::wait(100);    
        }
}
        

